@@ -12,7 +12,7 @@ using namespace std;
 
 // ISO-8859-1 HTML character entities
 
-static char * LATIN1[] = {
+static const char * LATIN1[] = {
 	"&Agrave;",
 	"&Atilde;",
 	"&Aring;",
@@ -46,6 +46,10 @@ static char * LATIN1[] = {
 
 #define LATIN1_LEN (sizeof(LATIN1) / sizeof(char *))
 
+#define ENTITY_APOS "&apos;"
+#define UTF8_APOS "\xe2\x80\x99"
+#define APOSTROPHE "'"
+
 TextParser::TextParser() {
 	init((char *) NULL);
 }
@@ -78,7 +82,7 @@ int TextParser::is_wordchar(char * w)
 	}
 }
 
-char * TextParser::get_latin1(char * s)
+const char * TextParser::get_latin1(char * s)
 {
 	if (s[0] == '&') {
 		unsigned int i = 0;
@@ -159,7 +163,7 @@ char * TextParser::get_line()
 
 char * TextParser::next_token()
 {
-	char * latin1;
+	const char * latin1;
 	
 	for (;;) {
 		switch (state)
@@ -177,6 +181,13 @@ char * TextParser::next_token()
 		case 1: // wordchar
 			if ((latin1 = get_latin1(line[actual] + head))) {
 				head += strlen(latin1);
+			} else if ((is_wordchar((char *) APOSTROPHE) || (is_utf8() && is_wordchar((char *) UTF8_APOS))) && line[actual][head] == '\'' &&
+					is_wordchar(line[actual] + head + 1)) {
+				head++;
+			} else if (is_utf8() && is_wordchar((char *) APOSTROPHE) && // add Unicode apostrophe to the WORDCHARS, if needed
+					strncmp(line[actual] + head, UTF8_APOS, strlen(UTF8_APOS)) == 0 &&
+					is_wordchar(line[actual] + head + strlen(UTF8_APOS))) {
+				head += strlen(UTF8_APOS) - 1;
 			} else if (! is_wordchar(line[actual] + head)) {
 				state = 0;
 				char * t = alloc_token(token, &head);
@@ -271,7 +282,8 @@ void TextParser::set_url_checking(int check)
 
 char * TextParser::alloc_token(int token, int * head)
 {
-    if (get_url(token, head)) return NULL;
+    int url_head = *head;
+    if (get_url(token, &url_head)) return NULL;
     char * t = (char *) malloc(*head - token + 1);
     if (t) {
         t[*head - token] = '\0';
